"""Define routes for Member Audit."""

from django.urls import path

from memberaudit.views import (
    admin,
    character_finder,
    character_viewer_1,
    character_viewer_2,
    characters,
    data_export,
    reports,
)

app_name = "memberaudit"

urlpatterns = [
    # characters
    path("", characters.index, name="index"),
    path("launcher", characters.launcher, name="launcher"),
    path("add_character", characters.add_character, name="add_character"),
    path(
        "remove_character/<int:character_pk>/",
        characters.remove_character,
        name="remove_character",
    ),
    path(
        "share_character/<int:character_pk>/",
        characters.share_character,
        name="share_character",
    ),
    path(
        "unshare_character/<int:character_pk>/",
        characters.unshare_character,
        name="unshare_character",
    ),
    # character viewer
    path(
        "character_viewer/<int:character_pk>/",
        character_viewer_1.character_viewer,
        name="character_viewer",
    ),
    path(
        "character_assets_data/<int:character_pk>/",
        character_viewer_1.character_assets_data,
        name="character_assets_data",
    ),
    path(
        "character_asset_container/<int:character_pk>/<int:parent_asset_pk>/",
        character_viewer_1.character_asset_container,
        name="character_asset_container",
    ),
    path(
        "character_asset_container_data/<int:character_pk>/<int:parent_asset_pk>/",
        character_viewer_1.character_asset_container_data,
        name="character_asset_container_data",
    ),
    path(
        "character_contacts_data/<int:character_pk>/",
        character_viewer_1.character_contacts_data,
        name="character_contacts_data",
    ),
    path(
        "character_contracts_data/<int:character_pk>/",
        character_viewer_1.character_contracts_data,
        name="character_contracts_data",
    ),
    path(
        "character_contract_details/<int:character_pk>/<int:contract_pk>/",
        character_viewer_1.character_contract_details,
        name="character_contract_details",
    ),
    path(
        "character_contract_items_included_data/<int:character_pk>/<int:contract_pk>/",
        character_viewer_1.character_contract_items_included_data,
        name="character_contract_items_included_data",
    ),
    path(
        "character_contract_items_requested_data/<int:character_pk>/<int:contract_pk>/",
        character_viewer_1.character_contract_items_requested_data,
        name="character_contract_items_requested_data",
    ),
    path(
        "character_corporation_history/<int:character_pk>/",
        character_viewer_1.character_corporation_history,
        name="character_corporation_history",
    ),
    path(
        "character_fw_stats/<int:character_pk>/",
        character_viewer_1.character_fw_stats,
        name="character_fw_stats",
    ),
    path(
        "character_implants_data/<int:character_pk>/",
        character_viewer_1.character_implants_data,
        name="character_implants_data",
    ),
    path(
        "character_loyalty_data/<int:character_pk>/",
        character_viewer_1.character_loyalty_data,
        name="character_loyalty_data",
    ),
    path(
        "character_jump_clones_data/<int:character_pk>/",
        character_viewer_2.character_jump_clones_data,
        name="character_jump_clones_data",
    ),
    path(
        "character_mail_headers_by_label_data/<int:character_pk>/<int:label_id>/",
        character_viewer_2.character_mail_headers_by_label_data,
        name="character_mail_headers_by_label_data",
    ),
    path(
        "character_mail_headers_by_list_data/<int:character_pk>/<int:list_id>/",
        character_viewer_2.character_mail_headers_by_list_data,
        name="character_mail_headers_by_list_data",
    ),
    path(
        "character_mail/<int:character_pk>/<int:mail_pk>/",
        character_viewer_2.character_mail,
        name="character_mail",
    ),
    path(
        "character_mining_ledger_data/<int:character_pk>/",
        character_viewer_2.character_mining_ledger_data,
        name="character_mining_ledger_data",
    ),
    path(
        "character_planets_data/<int:character_pk>/",
        character_viewer_2.character_planets_data,
        name="character_planets_data",
    ),
    path(
        "character_roles_data/<int:character_pk>/",
        character_viewer_2.character_roles_data,
        name="character_roles_data",
    ),
    path(
        "character_skillqueue_data/<int:character_pk>/",
        character_viewer_2.character_skillqueue_data,
        name="character_skillqueue_data",
    ),
    path(
        "character_skills_data/<int:character_pk>/",
        character_viewer_2.character_skills_data,
        name="character_skills_data",
    ),
    path(
        "character_skill_sets_data/<int:character_pk>/",
        character_viewer_2.character_skill_sets_data,
        name="character_skill_sets_data",
    ),
    path(
        "character_standings_data/<int:character_pk>/",
        character_viewer_2.character_standings_data,
        name="character_standings_data",
    ),
    path(
        "character_titles_data/<int:character_pk>/",
        character_viewer_2.character_titles_data,
        name="character_titles_data",
    ),
    path(
        "character_wallet_journal_data/<int:character_pk>/",
        character_viewer_2.character_wallet_journal_data,
        name="character_wallet_journal_data",
    ),
    path(
        "character_wallet_transactions_data/<int:character_pk>/",
        character_viewer_2.character_wallet_transactions_data,
        name="character_wallet_transactions_data",
    ),
    path(
        "character_skill_set_details/<int:character_pk>/<int:skill_set_pk>",
        character_viewer_2.character_skill_set_details,
        name="character_skill_set_details",
    ),
    path(
        "character_attributes_data/<int:character_pk>",
        character_viewer_1.character_attribute_data,
        name="character_attribute_data",
    ),
    # character finder
    path(
        "character_finder", character_finder.character_finder, name="character_finder"
    ),
    path(
        "character_finder_data",
        character_finder.CharacterFinderListJson.as_view(),
        name="character_finder_data",
    ),
    path(
        "character_finder_list_fdd_data",
        character_finder.character_finder_list_fdd_data,
        name="character_finder_list_fdd_data",
    ),
    # reports
    path("reports", reports.reports, name="reports"),
    path(
        "user_compliance_report_data",
        reports.user_compliance_report_data,
        name="user_compliance_report_data",
    ),
    path(
        "corporation_compliance_report_data",
        reports.corporation_compliance_report_data,
        name="corporation_compliance_report_data",
    ),
    path(
        "skill_sets_report_data",
        reports.skill_sets_report_data,
        name="skill_sets_report_data",
    ),
    # data export
    path("data-export/", data_export.data_export, name="data_export"),
    path(
        "data-export/run-update/<str:topic>",
        data_export.data_export_run_update,
        name="data_export_run_update",
    ),
    path(
        "data-export/download/<str:topic>",
        data_export.download_export_file,
        name="download_export_file",
    ),
    # admin
    path(
        "admin/create-skillset-from-fitting",
        admin.admin_create_skillset_from_fitting,
        name="admin_create_skillset_from_fitting",
    ),
    path(
        "admin/create-skill-set-from-skill-plan",
        admin.admin_create_skillset_from_skill_plan,
        name="admin_create_skillset_from_skill_plan",
    ),
]



from pathlib import Path
from tempfile import TemporaryDirectory
from unittest.mock import patch

from django.http import Http404
from django.test import RequestFactory, TestCase
from django.urls import reverse

from app_utils.testing import create_user_from_evecharacter

from memberaudit.tests.testdata.load_entities import load_entities
from memberaudit.views.data_export import (
    data_export,
    data_export_run_update,
    download_export_file,
)

MODULE_PATH = "memberaudit.views.data_export"


class TestDataExport(TestCase):
    @classmethod
    def setUpTestData(cls) -> None:
        cls.factory = RequestFactory()
        load_entities()

    def test_should_open_exports_page_with_permission(self):
        # given
        user, _ = create_user_from_evecharacter(
            1122, permissions=["memberaudit.basic_access", "memberaudit.exports_access"]
        )
        request = self.factory.get(reverse("memberaudit:data_export"))
        request.user = user
        # when
        response = data_export(request)
        # then
        self.assertEqual(response.status_code, 200)

    def test_should_not_open_exports_page_without_permission(self):
        # given
        user, _ = create_user_from_evecharacter(
            1122, permissions=["memberaudit.basic_access"]
        )
        request = self.factory.get(reverse("memberaudit:data_export"))
        request.user = user
        # when
        response = data_export(request)
        # then
        self.assertEqual(response.status_code, 302)

    @patch(MODULE_PATH + ".data_exporters.default_destination")
    def test_should_return_export_file(self, mock_default_destination):
        with TemporaryDirectory() as tmp_dir_name:
            # given
            contract_item_file = Path(tmp_dir_name) / "memberaudit_contract-item.zip"
            with contract_item_file.open(mode="w") as _:
                pass
            mock_default_destination.return_value = Path(tmp_dir_name)
            user, _ = create_user_from_evecharacter(
                1122,
                permissions=["memberaudit.basic_access", "memberaudit.exports_access"],
            )
            request = self.factory.get(
                reverse("memberaudit:download_export_file", args=["contract-item"])
            )
            request.user = user
            # when
            response = download_export_file(request, "contract-item")
            # then
            self.assertEqual(response.status_code, 200)

    @patch(MODULE_PATH + ".data_exporters.default_destination")
    def test_should_raise_404_when_export_file_not_found(
        self, mock_default_destination
    ):
        with TemporaryDirectory() as tmpdirname:
            # given
            mock_default_destination.return_value = Path(tmpdirname)
            user, _ = create_user_from_evecharacter(
                1122,
                permissions=["memberaudit.basic_access", "memberaudit.exports_access"],
            )
            request = self.factory.get(
                reverse("memberaudit:download_export_file", args=["contract-item"])
            )
            request.user = user
            # when/then
            with self.assertRaises(Http404):
                download_export_file(request, "contract-item")

    @patch(MODULE_PATH + ".messages")
    @patch(MODULE_PATH + ".tasks.export_data_for_topic")
    def test_should_start_export_task(
        self, mock_task_export_data_for_topic, mock_messages_plus
    ):
        # given
        user, _ = create_user_from_evecharacter(
            1122, permissions=["memberaudit.basic_access", "memberaudit.exports_access"]
        )
        request = self.factory.get(
            reverse("memberaudit:data_export_run_update", args=["contract-item"])
        )
        request.user = user
        # when
        response = data_export_run_update(request, "contract-item")
        # then
        self.assertEqual(response.status_code, 302)
        self.assertTrue(mock_task_export_data_for_topic.delay.called)
        _, kwargs = mock_task_export_data_for_topic.delay.call_args
        self.assertEqual(kwargs["topic"], "contract-item")
        self.assertEqual(kwargs["user_pk"], user.pk)
        self.assertTrue(mock_messages_plus.info.called)



# Generated by Django 4.0.10 on 2023-11-15 16:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("memberaudit", "0014_fix_default_permissions"),
    ]

    operations = [
        migrations.AddField(
            model_name="charactership",
            name="item_id",
            field=models.PositiveBigIntegerField(default=0),
            preserve_default=False,
        ),
    ]
